function [ V, X_KP, X_BT, S_Glc, S_Dex, S_Ac, S_Mal, O, S_Pro, S_Succ] = dmmm2_BtKp_two_CSTR_ss ( community, tspan, initialConditions)
%% dmmm2: dynamic multispecies metabolic modeling framework
%   community:  description of the microbial community and its environment
%       community.species:  species in the community
%       community.mets:     metabolites to be tracked by DMMM
%       community.Xfeed:    biomass in the feed (g/L)
%       community.Sfeed:    metabolites in the feed (mmol/L)
%       community.Fin:      flow rate in (L/hr)
%       community.Fout:     flowrate out (L/hr)


global SS_X_KP;
global SS_X_BT;
global SS_O;
global SS_Glc;
global SS_Dex;
global SS_Ac;
% New additions, Dec 2017:
global SS_Pro;      %Propionate
global SS_Succ;     %Succinate
global SS_Mal;      %Malate

numSpecies = length(community.species);
numMetabolites = length(community.mets);

disp(initialConditions)

%%  Setting initial conditions
%% Reminder initialConditions = [initV(1) initX(2) initS(6) initO(1)];
V_i = initialConditions(1);
X_i_KP = initialConditions(2);      % Biomass KP
X_i_BT = initialConditions(3);      % Biomass BT
S_i_Glc = initialConditions(4);     % Glucose Conc
S_i_Dex = initialConditions(5);     % Dex Conc
S_i_Ac = initialConditions(6);      % Acetate Conc
S_i_Mal = initialConditions(7);     % Malate Conc
S_i_Pro = initialConditions(8);     % Propionate Conc
S_i_Succ = initialConditions(9);    % Succinate Conc
O_i = initialConditions(10);        % Oxygen Conc


%% Defining Output for dmmm2_BtKP
global V;
global X_KP;
global X_BT;
global S_Glc;
global S_Dex;
global S_Ac;
global S_Mal;
global O;
global S_Pro;
global S_Succ;

V = [];
X_KP = [];
X_BT = [];
S_Glc = [];
S_Dex = [];
S_Ac = [];
S_Mal = [];
O = [];
S_Pro = [];
S_Succ = [];

V(1,1)=V_i;
X_KP(1,1)=X_i_KP;
X_BT(1,1)=X_i_BT;
S_Glc(1,1)=S_i_Glc;
S_Dex(1,1)=S_i_Dex;
S_Ac(1,1)=S_i_Ac;
S_Mal(1,1)=S_i_Mal;
O(1,1)=O_i;
S_Pro(1,1)=S_i_Pro;
S_Succ(1,1)=S_i_Succ;

V_f=0.2;        %Doesn't matter how it's defined here. 
X_f_KP=0;
X_f_BT=0;
S_f_Glc=0;
S_f_Dex=0;
S_f_Ac=0;
S_f_Mal=0;
O_f=0;
S_f_Pro=0;
S_f_Succ=0;

delta_t = tspan(2);

for i=1:length(tspan)-1;

    i;
    t = i*delta_t-delta_t;
    [vs, mu, vs_Ox, vs_Dex_Glu] = parameters_BtKp_two_CSTR(S_i_Glc,S_i_Dex,S_i_Ac,S_i_Mal,O_i,S_i_Pro,S_i_Succ,community,t);
           
 

    %% CHECKING OXYGEN TO MAKE SURE NOT NEGATIVE ****
    %%% If negative will recaculate parameters
    kla=47.4;                % units: 1/hr
    Total_Air_Flow = 50;     % ml/min
    Oxygen_Flow = community.Sfeed(3);         
    C_star = (Oxygen_Flow/Total_Air_Flow)*(6.056)/(0.2095)/(32); %6.056mg/L and 32mg/mmol -- % (1.75/50)*6.056/0.2/32
    O_feed = kla*(C_star-O_i)*delta_t;
    G_feed = community.Fin*(community.Sfeed(1)-S_i_Glc)*delta_t/V_i;

    KP_1 = V_i/(mu(1)*V_i-community.Fin);
    KP_2 = X_i_KP*exp((mu(1)-community.Fin/V_i)*delta_t);
    BT_1 = V_i/(mu(2)*V_i-community.Fin);
    BT_2 = X_i_BT*exp((mu(2)-community.Fin/V_i)*delta_t);
    
    % What is the new oxygen value?
    if (mu(1) == 0 && mu(2) ~= 0)       %mu(1) = mu for KP
        O_f = O_i + O_feed;
        S_f_Glc = S_i_Glc + (vs(2,1)*BT_1)*(BT_2-X_i_BT) + (vs_Dex_Glu*BT_1)*(BT_2-X_i_BT) + G_feed;
    elseif (mu(1) ~= 0 && mu(2) == 0)
        O_f = O_i + (vs_Ox*KP_1)*(KP_2-X_i_KP) + O_feed;
        S_f_Glc = S_i_Glc + (vs(1,1)*KP_1)*(KP_2-X_i_KP) + G_feed;
    elseif (mu(1) ~= 0 && mu(2) ~= 0)
        O_f = O_i + (vs_Ox*KP_1)*(KP_2-X_i_KP) + O_feed;
        S_f_Glc = S_i_Glc + (vs(1,1)*KP_1)*(KP_2-X_i_KP) + (vs(2,1)*BT_1)*(BT_2-X_i_BT) + (vs_Dex_Glu*BT_1)*(BT_2-X_i_BT) + G_feed;
    end
        
    % Set O_f=0 and recaculate vs_Ox
    if (O_f < 0 && S_f_Glc >=0)
        vs_Ox_new = (-O_i - O_feed)/((KP_2-X_i_KP)*KP_1);
        [vs, mu, vs_Ox, vs_Dex_Glu] = parameters_BtKp_two_CSTR_oxygen(S_i_Glc,S_i_Dex,S_i_Ac,S_i_Mal,O_i,S_i_Pro,S_i_Succ,community,t,vs_Ox_new);
    elseif (O_f >=0 && S_f_Glc < 0)
        vs_Glc_new = (-S_i_Glc - G_feed - (vs(2,1)*BT_1)*(BT_2-X_i_BT))/((KP_2-X_i_KP)*KP_1);
        [vs, mu, vs_Ox, vs_Dex_Glu] = parameters_BtKp_two_CSTR_glucose(S_i_Glc,S_i_Dex,S_i_Ac,S_i_Mal,O_i,S_i_Pro,S_i_Succ,community,t,vs_Glc_new);
    elseif (O_f < 0 && S_f_Glc < 0)
        vs_Ox_new = (-O_i - O_feed)/((KP_2-X_i_KP)*KP_1);
        vs_Glc_new = (-S_i_Glc - G_feed - (vs(2,1)*BT_1)*(BT_2-X_i_BT))/((KP_2-X_i_KP)*KP_1);
        [vs, mu, vs_Ox, vs_Dex_Glu] = parameters_BtKp_two_CSTR_glc_ox(S_i_Glc,S_i_Dex,S_i_Ac,S_i_Mal,O_i,S_i_Pro,S_i_Succ,community,t,vs_Ox_new,vs_Glc_new);
    end
    
    
    %% Calculating the new values
    V_f = V_i + (community.Fin - community.Fout)*delta_t;
    X_f_KP = X_i_KP * exp((mu(1)-community.Fin/V_i)*delta_t);
    X_f_BT = X_i_BT * exp((mu(2)-community.Fin/V_i)*delta_t);
    BT_1 = V_i/(mu(2)*V_i-community.Fin);
    BT_2 = X_i_BT*exp((mu(2)-community.Fin/V_i)*delta_t);
    G_feed = community.Fin*(community.Sfeed(1)-S_i_Glc)*delta_t/V_i;
    D_feed = community.Fin*(community.Sfeed(2)-S_i_Dex)*delta_t/V_i;
    %% NEW Oxygen Equation
    O_feed = kla*(C_star-O_i)*delta_t;
    A_feed = community.Fin*(community.Sfeed(4)-S_i_Ac)*delta_t/V_i;
    M_feed = community.Fin*(community.Sfeed(5)-S_i_Mal)*delta_t/V_i;
    KP_1 = V_i/(mu(1)*V_i-community.Fin);
    KP_2 = X_i_KP*exp((mu(1)-community.Fin/V_i)*delta_t);
    if (mu(1) == 0 && mu(2) ~= 0)   %If KP growth is zero
        S_f_Glc = S_i_Glc + (vs(2,1)*BT_1)*(BT_2-X_i_BT) + (vs_Dex_Glu*BT_1)*(BT_2-X_i_BT) + G_feed;
        S_f_Dex = S_i_Dex + (vs(2,2)*BT_1)*(BT_2-X_i_BT) + D_feed;
        S_f_Ac = S_i_Ac + (vs(2,3)*BT_1)*(BT_2-X_i_BT) + A_feed;
        S_f_Mal = S_i_Mal +  (vs(2,4)*BT_1)*(BT_2-X_i_BT) + M_feed ;
        O_f = O_i + O_feed;
    elseif (mu(1) ~= 0 && mu(2) == 0) %If BT growth is zero
        S_f_Glc = S_i_Glc + (vs(1,1)*KP_1)*(KP_2-X_i_KP) + G_feed; 
        S_f_Dex = S_i_Dex + D_feed;
        S_f_Ac = S_i_Ac + (vs(1,3)*KP_1)*(KP_2-X_i_KP) + A_feed;
        S_f_Mal = S_i_Mal + (vs(1,4)*KP_1)*(KP_2-X_i_KP) + M_feed;
        O_f = O_i + (vs_Ox*KP_1)*(KP_2-X_i_KP) + O_feed;
    elseif (mu(1) ~= 0 && mu(2) ~= 0)
        S_f_Glc = S_i_Glc + (vs(1,1)*KP_1)*(KP_2-X_i_KP) + (vs(2,1)*BT_1)*(BT_2-X_i_BT) + (vs_Dex_Glu*BT_1)*(BT_2-X_i_BT) + G_feed;
        S_f_Dex = S_i_Dex + (vs(2,2)*BT_1)*(BT_2-X_i_BT) + D_feed;
        S_f_Ac = S_i_Ac + (vs(1,3)*KP_1)*(KP_2-X_i_KP) + (vs(2,3)*BT_1)*(BT_2-X_i_BT) + A_feed;
        S_f_Mal = S_i_Mal + (vs(1,4)*KP_1)*(KP_2-X_i_KP) + (vs(2,4)*BT_1)*(BT_2-X_i_BT) + M_feed;
        O_f = O_i + (vs_Ox*KP_1)*(KP_2-X_i_KP) + O_feed;
    elseif (mu(1) == 0 && mu(2) == 0)
        S_f_Glc = S_i_Glc + G_feed;
        S_f_Dex = S_i_Dex + D_feed;
        S_f_Ac = S_i_Ac + A_feed;
        S_f_Mal = S_i_Mal + M_feed;
        O_f = O_i + O_feed;
    end
    

    % Saving these new values into Global variables
    V = [V V_f];
    X_KP = [X_KP X_f_KP];
    X_BT = [X_BT X_f_BT];
    S_Glc = [S_Glc S_f_Glc];
    S_Dex = [S_Dex S_f_Dex];
    S_Ac = [S_Ac S_f_Ac];
    S_Mal = [S_Mal S_f_Mal];
    O = [O O_f];
    
    
    global vs_KP_acetate;
    global vs_BT_acetate;
    global vs_KP_glucose;
    global vs_BT_glucose;
    global vs_KP_dextran;
    global vs_BT_dextran;
    vs_KP_glucose = [vs_KP_glucose vs(1,1)];
    vs_BT_glucose = [vs_BT_glucose vs(2,1)];
    vs_KP_dextran = [vs_KP_dextran vs(1,2)];
    vs_BT_dextran = [vs_BT_dextran vs(2,2)];
    vs_KP_acetate = [vs_KP_acetate vs(1,3)];
    vs_BT_acetate = [vs_BT_acetate vs(2,3)];
   
    % Resetting for the next time step
    V_i = V_f;
    X_i_KP = X_f_KP;
    X_i_BT = X_f_BT;
    S_i_Glc = S_f_Glc;
    S_i_Dex = S_f_Dex;
    S_i_Ac = S_f_Ac;
    S_i_Mal = S_f_Mal;
    O_i = O_f;
     
end

